/* returns a julian date in days from date and time*/ 
static PyObject *gdate(PyObject *self, PyObject *args) {
   double day, jd, F, A, B, C, D, E, G, I;
   int year, month;

   if (!PyArg_ParseTuple(args, "d", &jd)) {
      return NULL;
   }

    jd = jd + 0.5;
    F = modf(jd, &I);
    I = (int)(I);
    A = trunc((I - 1867216.25)/36524.25);
    if (I > 2299160) { B = I + 1 + A - trunc(A / 4.); }
    else { B = I; }
    C = B + 1524;
    D = trunc((C - 122.1) / 365.25);
    E = trunc(365.25 * D);
    G = trunc((C - E) / 30.6001);
    day = C - E + F - trunc(30.6001 * G);
    
    if (G < 13.5) { month = G - 1; }
    else { month = G - 13; }
        
    if (month > 2.5) { year = D - 4716; }
    else { year = D - 4715; }

   return Py_BuildValue("iid", year, month, day ); 
}

/* returns a local sidereal time and sidereal time in greenwich */ 
static PyObject *lst(PyObject *self, PyObject *args) {
   double lon, jday, jday0, dhour, t3, mst0, lst;
   
   if (!PyArg_ParseTuple(args, "dd", &lon, &jday)) {
      return NULL;
   }
   jday0 = (int)(jday - 0.5)+0.5;
/*   printf("%f\n",jday0); */
   dhour = (jday - jday0)*24; 
   t3 = (jday0-2451545)/36525;
   mst0 = 6.697374558 + 2400.05133691*t3 + 0.0000258622*pow(t3,2) - 0.0000000017*pow(t3,3);
   mst0 = ((mst0/24) - (int)(mst0/24))*24;
   if (mst0 < 0) {mst0 = mst0 + 24;}
   lst = (mst0 + dhour * 1.0027379093) + lon/15;

   return Py_BuildValue("dd", lst, mst0); 
}

/*returns a true anomaly and distance from sun for all kinds of excentricity*/
static PyObject *truear(PyObject *self, PyObject *args) {
   double k, q, e, dt, a, mm, e1, dee, e2, xv,yv, truea = 0, r = 0, peri, node, incl;
   double xh, yh, zh, gg, ww, aa1, aa2, aa3, cc, www, lonecl, latecl;
   
   k = 0.01720209895;
   
   if (!PyArg_ParseTuple(args, "dddddd", &q, &e, &dt, &peri, &node, &incl )) {
      return NULL;
   }

   peri = peri/180.*M_PI;
   node = node/180.*M_PI;
   incl = incl/180.*M_PI;
   
   if ( e < 0.9999 ) { /* ellipse solution for e under 1 */ 
       a = q / ( 1. - e );
       mm = dt * k / pow(a,1.5) ;
       e1 = mm + e * sin(mm) * (1. + e * cos(mm));
       dee = fabs(e1 - mm);
       do {
           e2 = e1 - (e1 - e * sin(e1) - mm) / (1. - e * cos(e1));
           dee = fabs(e2 - e1);
           e1 = e2;
       } while (dee > 0.000001);
       xv = a * ( cos(e1) - e );
       yv = a * sqrt(1. - e*e) * sin(e1);
       truea = 2.*M_PI + atan2( yv, xv );
       r = sqrt( xv*xv + yv*yv ); 
   }
   else if ( e == 1 ){ /* parabolic solution for e = 1 */
       mm = 1.5 * ( dt * ( k / sqrt(2.) ) / pow(q,1.5) );
       e1 = sqrt( 1. + mm*mm );
       dee = pow(e1+mm,1./3.) - pow(e1-mm,1./3.);
       truea = 2. * atan(dee);
       r = q * (1. + dee*dee);
   } 
   else if ( (e >= 0.9999) & (e != 1) ) { /* hyperbolic solution */
       mm = 0.75 * dt * k * sqrt( (1. + e) / pow(q,3.) );
       e1 = sqrt( 1 + mm*mm );
       ww = pow(e1+mm,1./3.) - pow(e1-mm,1./3.);
       dee = (1. - e) / (1. + e);
       aa1 = 2./3. + 2./5. * ww*ww;
       aa2 = (7./5.) + (33./35.) * ww*ww + (37./175.) * pow(ww,4.);
       aa3 = ww*ww * ( (432./175.) + (956./1125.) * ww*ww + (84./1575.) * pow(ww,4.) );
       cc = ww*ww / (1. + ww*ww);
       gg = dee * cc*cc;
       www = ww * ( 1. + dee * cc * ( aa1 + aa2*gg + aa3*gg*gg ) );
       truea = 2. * atan(www);        
       r = q * ( 1. + ww*ww ) / ( 1. + ww*ww * dee );
   } 

   xh = r * ( cos(node) * cos(truea+peri) - sin(node) * sin(truea+peri) * cos(incl) );
   yh = r * ( sin(node) * cos(truea+peri) + cos(node) * sin(truea+peri) * cos(incl) );
   zh = r * ( sin(truea+peri) * sin(incl) );
    
   lonecl = atan2( yh,xh );
   latecl = atan2( zh, sqrt(xh*xh+yh*yh) );

   return Py_BuildValue("ddd", r, lonecl, latecl ); 
}

/*returns a dt from heliocentric distance*/
static PyObject *finddt(PyObject *self, PyObject *args) {
   double k, r, dt = 0, q, e, a, eanom, mm;
   
   k = 0.01720209895;
   
   if (!PyArg_ParseTuple(args, "ddd", &r, &q, &e )) {
      return NULL;
   } 
   
   if ( e < 1 ) { /* ellipse solution for e under 1 */ 
       a = q / ( 1. - e );
       eanom = acos( (1 - r/a)/e );
       mm = eanom - e * sin(eanom); 
       dt = mm * pow(a,1.5) / k ;
   }
   else if ( e == 1 ){ /* parabolic solution for e = 1 */
       eanom = sqrt(r/q-1.);
       mm = eanom*eanom*eanom + 3*eanom;
       dt = (mm*q*sqrt(q))/0.0364911624;
   } 
   else if ( e > 1 ) { /* hyperbolic solution */
       a = q / ( 1. - e );
       eanom = acosh( ((q/(1.-e) - r)*(1.-e))/(q*e) );
       mm = e*sinh(eanom) - eanom;
       dt = sqrt(pow(-a,3)/k) * mm ;
       dt = -1.01279229 + dt * 7.62856729 + dt*dt * -3.02014611e-04 + dt*dt*dt * 2.13485966e-07 ; /* bulgarian constant */
   }

   return Py_BuildValue("d", dt ); 
}

/* returns mean anomaly, longnitude, distance and eclon */
static PyObject *sunpos(PyObject *self, PyObject *args) {
   double ddays, w, e, ecl, M, E, x, y, v, r, lon;

   if (!PyArg_ParseTuple(args, "d", &ddays )) {
      return NULL;
   }
   w = (282.9404 + 4.70935E-5 * ddays)/180*M_PI;
   e = 0.016709 - 1.151E-9 * ddays;
   ecl = (23.4393 - 3.563E-7 * ddays)/180*M_PI;
   M = (356.0470 + 0.9856002585 * ddays)/180*M_PI;
   
   E = M + e * sin(M) * ( 1. + e * cos(M) );

   x = cos(E) - e;
   y = sin(E) * sqrt(1. - e*e);

   v = atan2( y, x );
   r = sqrt(x*x + y*y);
   lon = v + w;

   return Py_BuildValue("dddd", M, lon, r, ecl ); 
}

/* sol long and sun dist to coords */
static PyObject *suncoords(PyObject *self, PyObject *args) {
   double lonsun, rs, xs, ys;

   if (!PyArg_ParseTuple(args, "dd", &lonsun, &rs )) {
      return NULL;
   }
   
   xs = rs * cos(lonsun);
   ys = rs * sin(lonsun);

   return Py_BuildValue("dd", xs, ys ); 
}

/* giving sun ra and de */
static PyObject *sunrade(PyObject *self, PyObject *args) {
   double xs, ys, ecl, x, y, z, ra, de;

   if (!PyArg_ParseTuple(args, "ddd", &xs, &ys, &ecl )) {
      return NULL;
   }

    x = xs;
    y = ys * cos(ecl);
    z = ys * sin(ecl);

    if (atan2( y, x ) < 0) {
        ra = (atan2( y, x )+2.*M_PI)/M_PI*180./15.;
    }
    else {
        ra = (atan2( y, x ))/M_PI*180./15.;
    }
    de = (atan2( z, sqrt(x*x+y*y) ))/M_PI*180.;

   return Py_BuildValue("dd", ra, de ); 
}

/* ra and dec from heliocentric coords */
static PyObject *objradec(PyObject *self, PyObject *args) {
   double r, lonecl, latecl, xs, ys, ecl;
   double xhc, yhc, zhc, xg, yg, zg, xe, ye, ze, ra, de, dist;

   if (!PyArg_ParseTuple(args, "dddddd", &r, &lonecl, &latecl, &xs, &ys, &ecl )) {
      return NULL;
   }

   xhc = r * cos(lonecl) * cos(latecl);
   yhc = r * sin(lonecl) * cos(latecl);
   zhc = r * sin(latecl);
    
   xg = xhc + xs;
   yg = yhc + ys;
   zg = zhc;
    
   xe = xg;
   ye = yg * cos(ecl) - zg * sin(ecl);
   ze = yg * sin(ecl) + zg * cos(ecl);
        
   if (atan2( ye, xe ) < 0) {
      ra = (atan2( ye, xe )+2*M_PI)/M_PI*180./15.;
   }
   else {
      ra = (atan2( ye, xe ))/M_PI*180./15.;
   }
   de = (atan2( ze, sqrt(xe*xe+ye*ye) ))/M_PI*180.;
    
   dist = sqrt( xg*xg + yg*yg + zg*zg );

   return Py_BuildValue("ddd", ra, de, dist ); 
}

/* best time */
static PyObject *transit(PyObject *self, PyObject *args) {
   double gmst0, longn, ra, tz, tv;
   double transit, btime, dt1, dt2; 

   if (!PyArg_ParseTuple(args, "ddddd", &gmst0, &longn, &ra, &tz, &tv )) {
      return NULL;
   }
   btime = 0;

   transit = ((ra*15. - gmst0 - longn) / 15.04107)/24.;
   if ( transit < 0 ) { transit = (transit - (int)(transit) + 1.) * 24.; }
   else { transit = (transit - (int)(transit)) * 24.; }

   if (tz < tv) {
     if ( (transit>tz) && (transit<tv) ) { btime = transit; }
     else if ( transit<tz ) { 
       dt1 = tz - transit;
       dt2 = transit + (24.-tv); 
       if (dt1 < dt2) { btime = tz; }
       else { btime = tv; }
       }
     else if ( transit>tv ) { 
       dt1 = tz + (24.-transit);
       dt2 = transit - tv; 
       if (dt1 < dt2) { btime = tz; }
       else { btime = tv; }
       }
     }
   else if (tz > tv) {
     if ( (transit>tz) || (transit<tv) ) { 
       btime = transit; 
       }
     else { 
       dt1 = tz - transit;
       dt2 = transit - tv; 
       if (dt1 < dt2) { 
         btime = tz; 
         }
       else { 
         btime = tv; 
         }
       }
     }
   else { btime = 0; }

   return Py_BuildValue("d", btime ); 
}

/* rise set */
static PyObject *riseset(PyObject *self, PyObject *args) {
   double lonsun, ras, des, sunalt, lat, longn, tz, tv;
   double gmst0, utsuns, coslha, lha; 

   if (!PyArg_ParseTuple(args, "dddddd", &lonsun, &ras, &des, &longn, &lat, &sunalt )) {
      return NULL;
   }

   gmst0 = (lonsun)/M_PI*180. + 180.;
   utsuns = ((ras*15. - gmst0 - longn) / 15.)/24.;
   if ( utsuns < 0 ) { utsuns = (utsuns - (int)(utsuns) + 1.) * 24.; }
   else { utsuns = (utsuns - (int)(utsuns)) * 24.; }
   coslha = (sin((-sunalt)/180.*M_PI)-sin((lat)/180.*M_PI)*sin((des)/180.*M_PI) ) / ( cos((lat)/180.*M_PI)*cos((des)/180.*M_PI) );
   if ( (-1 <= coslha) && ( coslha <= 1) ) {
     lha = (acos(coslha))/M_PI*180./ 15.04107;
     tz = utsuns + lha; 
     tv = utsuns - lha; }
   else { tz = utsuns+12.; tv = tz; }

   return Py_BuildValue("dd", tz, tv ); 
}

static PyObject *objaltaaz(PyObject *self, PyObject *args) {
   double lst, ra, de, lat;
   double ha, ax, ay, az, xhor, yhor, zhor, alt, ref;

   if (!PyArg_ParseTuple(args, "dddd", &lst, &ra, &de, &lat )) {
      return NULL;
   }
   ha = lst-ra;     
   ha = (((ha + 12.)/24.)-(int)((ha + 12.)/24.))*24. - 12.;
    
   ha = (ha*15.)/180.*M_PI;
   de = (de)/180.*M_PI;
   lat = (lat)/180.*M_PI;
    
   ax = cos(ha)*cos(de);
   ay = sin(ha)*cos(de);
   az = sin(de);
    
   xhor = ax * sin(lat) - az * cos(lat);
   yhor = ay;
   zhor = ax * cos(lat) + az * sin(lat);
    
   if (atan2( yhor, xhor ) < 0) { az  = (atan2( yhor, xhor ))/M_PI*180. +360.; }
   else { az  = (atan2( yhor, xhor ))/M_PI*180.; }
   alt = (atan2( zhor, sqrt(xhor*xhor+yhor*yhor) ))/M_PI*180.;

   if (-20. < alt ) {
      ref = 1.02/tan( ( alt + 10.3 / (alt + 5.11) )/180.*M_PI );
      alt = alt + ref/60.;
   }

   return Py_BuildValue("dd", az, alt ); 
}

static PyObject *moonpos(PyObject *self, PyObject *args) {
   double ddays, ecl;
   double N, i, w, a, e, M, e1, dee, e2, x, y, v, xh, yh, zh, Ms, Ls, Lm, D, F; 
   double lo1, la1, di1, xhc, yhc, zhc, xe, ye, ze;
   double lonecl, latecl, ra, de, r;

   if (!PyArg_ParseTuple(args, "dd", &ddays, &ecl )) {
      return NULL;
   }
   N = (125.1228 - 0.0529538083 * ddays)/180.*M_PI;
   i = (5.1454)/180.*M_PI;
   w = (318.0634 + 0.1643573223 * ddays)/180.*M_PI;
   a = 60.2666;
   e = 0.054900;
   M = (115.3654 + 13.0649929509 * ddays)/180.*M_PI;
    
   e1 = M + e * sin(M);
   dee = fabs(e1 - M);
   do {
      e2 = M + e * sin(e1);
      dee = fabs(e2 - e1);
      e1 = e2;
   } while (dee > 0.0001);

   x = a * (cos(e1) - e);
   y = a * sqrt(1. - e*e) * sin(e1);
    
   r = sqrt( x*x + y*y );
   v = atan2( y, x );

   xh = r * ( cos(N) * cos(v+w) - sin(N) * sin(v+w) * cos(i) );
   yh = r * ( sin(N) * cos(v+w) + cos(N) * sin(v+w) * cos(i) );
   zh = r * sin(v+w) * sin(i);

   lonecl = atan2( yh, xh );
   latecl = atan2( zh, sqrt(xh*xh+yh*yh) );
        
   Ms = (356.0470 + 0.9856002585 * ddays)/180.*M_PI;
   Ls = (282.9404 + 4.70935E-5 * ddays)/180.*M_PI + Ms;
   Lm = N + w + M; 
   D = Lm - Ls;
   F = Lm - N;
    
   lo1 = -1.274 * sin(M - 2*D) +0.658 * sin(2*D) -0.186 * sin(Ms) -0.059 * sin(2*M - 2*D) -0.057 * sin(M - 2*D + Ms)+0.053 * sin(M + 2*D) +0.046 * sin(2*D - Ms) +0.041 * sin(M - Ms) -0.035 * sin(D) -0.031 * sin(M + Ms) -0.015 * sin(2*F - 2*D) +0.011 * sin(M - 4*D);
   la1 = -0.173 * sin(F - 2*D) -0.055 * sin(M - F - 2*D) -0.046 * sin(M + F - 2*D) +0.033 * sin(F + 2*D) +0.017 * sin(2*M + F);
    
   di1 = -0.58 * cos(M - 2*D) -0.46 * cos(2*D);
   lonecl = lonecl + (lo1)/180.*M_PI;
   latecl = latecl + (la1)/180.*M_PI;
   r = r + di1;

   xhc = cos(lonecl) * cos(latecl);
   yhc = sin(lonecl) * cos(latecl);
   zhc = sin(latecl);

   xe = xhc;
   ye = yhc * cos(ecl) - zhc * sin(ecl);
   ze = yhc * sin(ecl) + zhc * cos(ecl);
    
   if (atan2( ye, xe ) < 0) { ra = (atan2( ye, xe )+2*M_PI)/M_PI*180./15.; }
   else { ra = (atan2( ye, xe ))/M_PI*180./15.; }
   de = (atan2( ze, sqrt(xe*xe+ye*ye) ))/M_PI*180.;

   return Py_BuildValue("ddddd", lonecl, latecl, ra, de, r ); 
}

static PyObject *sunradec(PyObject *self, PyObject *args) {
   double r, lon, ecl;
   double xe,ye,ze;
   double ra, de;

   if (!PyArg_ParseTuple(args, "ddd", &r, &lon, &ecl )) {
      return NULL;
   }

   xe = r * cos(lon);
   ye = r * sin(lon) * cos(ecl);
   ze = r * sin(lon) * sin(ecl);
    
   if (atan2( ye, xe ) < 0) { ra = (atan2( ye, xe )+2*M_PI)/M_PI*180./15.; }
   else { ra = (atan2( ye, xe ))/M_PI*180./15.; }
   de = (atan2( ze, sqrt(xe*xe+ye*ye) ))/M_PI*180.;

   return Py_BuildValue("dd", ra, de ); 
}

static PyObject *normradec(PyObject *self, PyObject *args) {
   double ra,de,jday;
   double t3, fi, zi, ro, x, y, z;
   double a1,a2,a3,b1,b2,b3,c1,c2,c3;

   if (!PyArg_ParseTuple(args, "ddd", &ra, &de, &jday )) {
      return NULL;
   }

   t3 = (jday - 2451545.)/36525.;
   
   fi = (0.6406161*t3 + 0.0000839*t3*t3 + 0.0000050*pow(t3,3.))/180.*M_PI; 
   zi = (0.6406161*t3 + 0.0003041*t3*t3 + 0.0000051*pow(t3,3.))/180.*M_PI;
   ro = (0.5567530*t3 - 0.0001185*t3*t3 - 0.0000116*pow(t3,3.))/180.*M_PI;

   a1 = cos(fi)*cos(ro)*cos(zi)-sin(fi)*sin(zi); 
   a2 = -sin(fi)*cos(ro)*cos(zi)-cos(fi)*sin(zi);
   a3 = -sin(ro)*cos(zi);
   b1 = cos(fi)*cos(ro)*sin(zi)+sin(fi)*cos(zi);
   b2 = -sin(fi)*cos(ro)*sin(zi)+cos(fi)*cos(zi);
   b3 = -sin(ro)*sin(zi); 
   c1 = cos(fi)*sin(ro);
   c2 = -sin(fi)*sin(ro);
   c3 = cos(ro);                

   x = cos((ra*15.)/180.*M_PI) * cos((de)/180.*M_PI);
   y = sin((ra*15.)/180.*M_PI) * cos((de)/180.*M_PI);
   z = sin((de)/180.*M_PI);

   x = x*a1 + y*b1 + z*c1;
   y = x*a2 + y*b2 + z*c2;
   z = x*a3 + y*b3 + z*c3;

   if (atan2( y, x ) < 0) { ra = (atan2( y, x )+2*M_PI)/M_PI*180./15.; }
   else { ra = (atan2( y, x ))/M_PI*180./15.; }
   de = (atan2( z, sqrt(x*x+y*y) ))/M_PI*180.;

   return Py_BuildValue("dd", ra, de ); 
}

static PyObject *appradec(PyObject *self, PyObject *args) {
   double ra,de,jday;
   double t3, fi, zi, ro, x, y, z;
   double a1,a2,a3,b1,b2,b3,c1,c2,c3;

   if (!PyArg_ParseTuple(args, "ddd", &ra, &de, &jday )) {
      return NULL;
   }

   t3 = (jday - 2451545.)/36525.;
    
   fi = (0.6406161*t3 + 0.0000839*t3*t3 + 0.0000050*pow(t3,3.))/180.*M_PI; 
   zi = (0.6406161*t3 + 0.0003041*t3*t3 + 0.0000051*pow(t3,3.))/180.*M_PI;
   ro = (0.5567530*t3 - 0.0001185*t3*t3 - 0.0000116*pow(t3,3.))/180.*M_PI;
   
   a1 = cos(fi)*cos(ro)*cos(zi)-sin(fi)*sin(zi); 
   a2 = -sin(fi)*cos(ro)*cos(zi)-cos(fi)*sin(zi);
   a3 = -sin(ro)*cos(zi);
   b1 = cos(fi)*cos(ro)*sin(zi)+sin(fi)*cos(zi);
   b2 = -sin(fi)*cos(ro)*sin(zi)+cos(fi)*cos(zi);
   b3 = -sin(ro)*sin(zi); 
   c1 = cos(fi)*sin(ro);
   c2 = -sin(fi)*sin(ro);
   c3 = cos(ro);                
 
   x = cos((ra*15.)/180.*M_PI) * cos((de)/180.*M_PI);
   y = sin((ra*15.)/180.*M_PI) * cos((de)/180.*M_PI);
   z = sin((de)/180.*M_PI);

   x = x*a1 + y*a2 + z*a3;
   y = x*b1 + y*b2 + z*b3;
   z = x*c1 + y*c2 + z*c3;

   if (atan2( y, x ) < 0) { ra = (atan2( y, x )+2*M_PI)/M_PI*180./15.; }
   else { ra = (atan2( y, x ))/M_PI*180./15.; }
   de = (atan2( z, sqrt(x*x+y*y) ))/M_PI*180.;

   return Py_BuildValue("dd", ra, de ); 
}

static PyObject *phasec(PyObject *self, PyObject *args) {
   double rs,r,dist;
   double phasea,phasec,gf,gb,kk,d2g,scangle,scafc,forwscac;

   if (!PyArg_ParseTuple(args, "ddd", &rs, &r, &dist )) {
      return NULL;
   }

   phasea = acos( ( r*r + dist*dist - rs*rs ) / (2*r*dist) )/M_PI*180.;
   phasec = 2.5 * log10(1.05-(0.06*phasea)+(0.00289*phasea*phasea)-(0.0000781*phasea*phasea*phasea)+(0.00000112*pow(phasea,4.))-(0.00000000777*pow(phasea,5.))+(2.11E-11*pow(phasea,6)));
   gf = 0.9;
   gb = -0.6;
   kk = 0.95;
   d2g = 1.0;
   scangle = 180. - phasea;
   scafc = d2g/(1.+d2g)*(kk*pow((1.+gf*gf)/(1.+gf*gf-2.*gf*cos(scangle/180.*M_PI)),1.5)+(1.-kk)*pow((1.+gb*gb)/(1.+gb*gb-2.*gb*cos(scangle/180.*M_PI)),1.5)+1./d2g);
   forwscac = 2.5*log10(scafc);

   return Py_BuildValue("dd", phasec, forwscac ); 
}

static PyObject *extcalc(PyObject *self, PyObject *args) {
   double alth, elevation, ext0, extinct, extcorr;

   if (!PyArg_ParseTuple(args, "ddd", &extinct, &alth, &elevation )) {
      return NULL;
   }

   extinct = extinct * pow(0.51,-1.3);
   ext0 = 0.1451 * exp((elevation/-1000.)/7.996) + extinct * exp((elevation/-1000.)/1.5) + 0.016;
   extcorr = ext0*(1./(cos((90.-alth)/180.*M_PI) + 0.025 * exp(-11.*cos((90.-alth)/180.*M_PI))));

   return Py_BuildValue("d", extcorr ); 
}

static PyObject *phasemag(PyObject *self, PyObject *args) {
   double phasea, phasemag;

   if (!PyArg_ParseTuple(args, "d", &phasea )) {
      return NULL;
   }
   phasemag = 2.5 * log10(1.05-(0.06*phasea)+(0.00289*pow(phasea,2.))-(0.0000781*pow(phasea,3.))+(0.00000112*pow(phasea,4.))-(0.00000000777*pow(phasea,5.))+(2.11E-11*pow(phasea,6.)));
   return Py_BuildValue("d", phasemag ); 
}

static PyObject *fwscmag(PyObject *self, PyObject *args) {
   double phasea, d2g, gf, gb, kk;
   double scangle, scafc, fwscmag;

   if (!PyArg_ParseTuple(args, "ddddd", &phasea, &d2g, &gf, &gb, &kk )) {
      return NULL;
   }
   scangle = (180. - phasea)/180.*M_PI;
   scafc = d2g/(1.+d2g)*(kk*pow((1.+gf*gf)/(1.+gf*gf-2.*gf*cos(scangle)),3.0/2.0)+(1.-kk)*pow((1.+gb*gb)/(1.+gb*gb-2.*gb*cos(scangle)),3.0/2.0)+1./d2g);
   fwscmag = -2.5 * log10(scafc);

   return Py_BuildValue("d", fwscmag ); 
}

static PyObject *centr(PyObject *self, PyObject *args) {
   double x,xs,x0,y,ys,y0;
   double dx,dy,c;

   if (!PyArg_ParseTuple(args, "dddd", &x,&x0,&y,&y0 )) {
      return NULL;
   }
   xs = 0; 
   ys = 0; 

   if ((x==x0) && (y==y0))         { xs =  0. ; ys =  0. ; } 
   else if ((x > x0) && (y > y0))  { xs =  0.5; ys =  0.5; } /* kvadrant 1:*/
   else if ((x > x0) && (y == y0)) { xs =  0.5; ys =  0. ; } /* intermezzo 1:*/
   else if ((x > x0) && (y < y0))  { xs =  0.5; ys = -0.5; } /* kvadrant 2:*/
   else if ((x == x0) && (y < y0)) { xs =  0. ; ys = -0.5; }/* intermezzo 2:*/
   else if ((x < x0) && (y < y0))  { xs = -0.5; ys = -0.5; } /* kvadrant 3:*/
   else if ((x < x0) && (y == y0)) { xs = -0.5; ys =  0. ; } /* intermezzo 3:*/
   else if ((x < x0) && (y > y0))  { xs = -0.5; ys =  0.5; } /* kvadrant 4:*/
   else if ((x == x0) && (y > y0)) { xs =  0. ; ys =  0.5; } /* intermezzo 4:*/

   dx = (x+xs) - x0;
   dy = (y+ys) - y0;
   c = sqrt(dx*dx + dy*dy);

   return Py_BuildValue("ddd", c, xs, ys ); 
}

static PyObject *centrin(PyObject *self, PyObject *args) {
   double x,xs,x0,y,ys,y0;
   double dx,dy,c;

   if (!PyArg_ParseTuple(args, "dddddd", &x,&x0,&xs,&y,&y0,&ys )) {
      return NULL;
   }

   dx = (x-xs) - x0;
   dy = (y-ys) - y0;
   c = sqrt(dx*dx + dy*dy);

   return Py_BuildValue("d", c ); 
}

static PyObject *centrout(PyObject *self, PyObject *args) {
   double x0,y0,i,j;
   double dx,dy,c;

   if (!PyArg_ParseTuple(args, "dddd", &x0,&y0,&i,&j )) {
      return NULL;
   }

   dx = x0*10. - i;
   dy = y0*10. - j;
   c = sqrt(dx*dx + dy*dy);

   return Py_BuildValue("d", c ); 
}

static PyObject *pythc(PyObject *self, PyObject *args) {
   double a,b,c;

   if (!PyArg_ParseTuple(args, "dd", &a, &b )) {
      return NULL;
   }
   
   c = sqrt( a*a + b*b );

   return Py_BuildValue("d", c ); 
}

static PyObject *pytha(PyObject *self, PyObject *args) {
   double a,b,c;

   if (!PyArg_ParseTuple(args, "dd", &b, &c )) {
      return NULL;
   }
   
   a = sqrt( c*c - b*b );

   return Py_BuildValue("d", a ); 
}

/*static PyObject *func(PyObject *self, PyObject *args) {
   double ;

   if (!PyArg_ParseTuple(args, "d", & )) {
      return NULL;
   }

   return Py_BuildValue("d",  ); 
}
*/
/*   printf("%f\n",jday0); */
/* METHODS */
static PyMethodDef astrolib_methods[] = {
   { "jday", jday, METH_VARARGS, NULL },
   { "gdate", gdate, METH_VARARGS, NULL },
   { "lst", lst, METH_VARARGS, NULL },
   { "truear", truear, METH_VARARGS, NULL },
   { "finddt", finddt, METH_VARARGS, NULL },
   { "sunpos", sunpos, METH_VARARGS, NULL },
   { "suncoords", suncoords, METH_VARARGS, NULL },
   { "sunrade", sunrade, METH_VARARGS, NULL },
   { "objradec", objradec, METH_VARARGS, NULL },
   { "transit", transit, METH_VARARGS, NULL },
   { "riseset", riseset, METH_VARARGS, NULL },
   { "objaltaaz", objaltaaz, METH_VARARGS, NULL },
   { "moonpos", moonpos, METH_VARARGS, NULL },
   { "sunradec", sunradec, METH_VARARGS, NULL },
   { "appradec", appradec, METH_VARARGS, NULL },
   { "normradec", normradec, METH_VARARGS, NULL },
   { "phasec", phasec, METH_VARARGS, NULL },
   { "extcalc", extcalc, METH_VARARGS, NULL },
   { "phasemag", phasemag, METH_VARARGS, NULL },
   { "fwscmag", fwscmag, METH_VARARGS, NULL },
   { "centr", centr, METH_VARARGS, NULL },
   { "centrin", centrin, METH_VARARGS, NULL },
   { "centrout", centrout, METH_VARARGS, NULL },
   { "pythc", pythc, METH_VARARGS, NULL },
   { "pytha", pytha, METH_VARARGS, NULL },
   { NULL, NULL, 0, NULL }
};

/* INIT */
void initalb(void) {
   Py_InitModule3("alb", astrolib_methods, NULL);
}

