#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import copy
import koprhand
import koprfunc
import koprclass
import koprdial
import koprfig
import datavar
import npfunc
import datetime
import setupobs
from PyQt5 import QtCore, QtGui, QtWidgets
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar

font = datavar.DefaultFont()

## *******************************************
############## WIDGET ################
## *******************************************
class Analyzer(QtWidgets.QWidget):
    
    def __init__(self, parent=None):
        super(Analyzer, self).__init__()
        self.initUI()
        
    def initUI(self):
#                         0  1  2  3   4  5  6  7  8   9  10  11              
        self.settings = [ 0, 0, 0, 1., 0, 0, 0, 0, 1., 0, 4., 0 ] # default settings
        self.peridict = {} # dictionary of returns
        self.returns = [] # list of returns containing [ year, on/off, color ]
        self.types = [['ccd', 1, 'o,circle'], ['vis', 1, 's,square'], ['mpct', 1, 'v,triangle_down'], ['mpcn', 1, '^,triangle_up']]
        self.obscode = [] # list of observers containing [ obscode, on/off ]
        self.obs_list = [] # observations array
        self.working_list = [] # observations working list array

        # Input grid
        grid = QtWidgets.QGridLayout()
        
        btnnew = QtWidgets.QPushButton('Select comet', self) 
        btnnew.clicked.connect(self.button_click_clear)

        btncobs = QtWidgets.QPushButton('Load COBS data', self)
        btncobs.clicked.connect(self.button_load_cobs) 
        
        btnmpc = QtWidgets.QPushButton('Load MPC data', self)
        btnmpc.clicked.connect(self.button_load_mpc)
        
        btnobs = QtWidgets.QPushButton('Open local data', self)
        btnobs.clicked.connect(self.button_openf)
        
        btnset = QtWidgets.QPushButton('Settings and selections', self)
        btnset.clicked.connect(self.button_click_setup)
        
        elements = [ btnnew, '', btncobs, btnmpc, btnobs, btnset ]
        for i in range(6):
          if i == 1:
            label = QtWidgets.QLabel('Comet name')
            label.setFont(font)
            label.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            grid.addWidget(label, i, 0 )
            self.comlbl = QtWidgets.QLineEdit(self)
            self.comlbl.setFont(font)
            self.comlbl.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            self.comlbl.setDisabled(1)
            grid.addWidget(self.comlbl, i, 1 )
          else:
            elements[i].setFont(font)
            elements[i].setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            grid.addWidget(elements[i], i, 0, 1, 2 )
            
        label = QtWidgets.QLabel('Regular formula')
        label.setFont(font)
        grid.addWidget(label, 6, 0 )

        btnsave = QtWidgets.QPushButton('Save', self) 
        btnsave.setFont(font)
        btnsave.clicked.connect(self.button_click_fsave)
        grid.addWidget(btnsave, 6, 1 )

        self.reg = QtWidgets.QTextEdit(self)
        self.reg.setFont(font) 
        self.reg.setMaximumHeight(btnnew.sizeHint().height() * 5)
        grid.addWidget(self.reg, 7, 0, 5, 2 )

        label = QtWidgets.QLabel('Additional formula')
        label.setFont(font)
        grid.addWidget(label, 12, 0 )

        btncalc = QtWidgets.QPushButton('Analyse', self) 
        btncalc.setFont(font)
        btncalc.clicked.connect(self.button_click_analyse)
        grid.addWidget(btncalc, 12, 1 )

        self.adit = QtWidgets.QTextEdit(self)
        self.adit.setFont(font) 
        self.adit.setMaximumHeight(btnnew.sizeHint().height() * 5)
        grid.addWidget(self.adit, 13, 0, 5, 2 )
        
        vboxl = QtWidgets.QVBoxLayout()
        vboxl.addLayout(grid)
        vboxl.addStretch(1)

        plotgrid = QtWidgets.QGridLayout()
        self.plotopt = [ [ QtWidgets.QLabel('Axis x ranges'), QtWidgets.QLineEdit(self), QtWidgets.QLineEdit(self), QtWidgets.QComboBox(self), QtWidgets.QCheckBox('Compare', self) ],
                         [ QtWidgets.QLabel('Axis y ranges'), QtWidgets.QLineEdit(self), QtWidgets.QLineEdit(self), QtWidgets.QComboBox(self), QtWidgets.QPushButton('Refresh', self) ] ]
        
        for i, line in enumerate(self.plotopt):
          for j, item in enumerate(self.plotopt[i]):
            self.plotopt[i][j].setFont(font)
            if j == 3:
              self.plotopt[i][j].setStyleSheet("selection-color: black")
            plotgrid.addWidget(self.plotopt[i][j], i, j) 

        for i, item in enumerate( ['Date','Days from T','r','Log(r)'] ):
          self.plotopt[0][3].addItem(item)
        self.plotopt[0][3].activated[str].connect(self.onActivatedDateR)
        self.dater = self.plotopt[0][3].currentIndex() 
        
        for i, item in enumerate( ['mag','mag - 5*log(dist)'] ):
          self.plotopt[1][3].addItem(item)
        self.plotopt[1][3].activated[str].connect(self.onActivatedMagR)
        
        self.plotopt[1][4].clicked.connect(self.button_click_update)
        
        self.sc = koprfig.LightCurve(self)
        self.sc.ax.grid(True)
        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)

        vboxr = QtWidgets.QVBoxLayout()
        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(plotgrid)
        hbox.addStretch(1)
        vboxr.addLayout(hbox)
        vboxr.addWidget(self.mpl_toolbar)
        vboxr.addWidget(self.sc)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr) 
        
        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - Analyzer')    
        self.show()        

### ************ NEW FUNCTIONS *********************
    def button_openf(self):
        if self.comlbl.text() == '':
          koprdial.InfoDialog(1, 'No comet loaded', 'Load comet first!' )             
        else:
          try:
            obs_file = open(koprhand.cfile(str(self.comlbl.text())), 'r', encoding="utf-8")
            obs_lines = obs_file.read()
            obs_file.close()
            a = koprdial.ICQData( obs_lines, str(self.comlbl.text()) + ' local data' )
            a.exec_()            
            if a.output != obs_lines:
              obs_file = open(koprhand.cfile(str(self.comlbl.text())), 'w', encoding="utf-8")
              obs_file.write(a.output)
              obs_file.close()              
          except IOError:
            obs_lines = []
    
    def button_load_cobs(self):
        self.load_data('cobs')

    def button_load_mpc(self):
        try:
          self.load_data('mpc')
        except Exception as e:
          if str(e).startswith('invalid literal for int'):
            koprdial.InfoDialog(1, 'No magnitude data on MPC', 'There are no magnitude data for this comet on MPC!')
          else:
            print(e)

    def load_data(self, loc):
        if self.comlbl.text() == '':
          koprdial.InfoDialog(1, 'No comet loaded', 'Load comet first!' )             
        else:
          elements = koprhand.loadcometid(str(self.comlbl.text()))
          obs_array = []

          try:
            obs_file = open(koprhand.cfile(self.kometa.destig), 'r', encoding="utf-8")
            obs_lines = obs_file.readlines()
            obs_file.close()
          except IOError:
            obs_lines = []

          if loc == 'cobs':
            filtobs = koprhand.loadcobscomet(self.kometa.destig)
          elif loc == 'mpc':
            perihelions, obslist = koprhand.loadmpccomet(str(self.comlbl.text()), self.kometa)  
            a = MPCFilter( perihelions, obslist, self.kometa )
            a.exec_() 
            filtobs = a.output

          results = koprhand.ObsResults(filtobs + obs_lines, self.kometa.destig, obs_array)
          
          koprdial.InfoDialog(2, 'Magnitude data loaded', 'There was totaly '+str(len(filtobs))+' observations found, total number of observations are '+str(len(results))+' after duplicates removed.' )
          self.UpdatePlot(True)

    def button_click_fsave(self):
        if str(self.reg.toPlainText()) == '' or self.comlbl.text() == '':
            koprdial.InfoDialog(1, 'Errors were encountered', 'Fill all required fields!')
        else:
            linelist = str(self.reg.toPlainText()).split('\n')
            linelist = [_f for _f in linelist if _f]
            destig = str(self.comlbl.text())
            addedline = ''
            for i in range (len(linelist)):
                linenew = linelist[i].replace(' ','').replace('H0=','').replace('mag','').replace('n=','').replace(']','')
                h0 = linenew.split(',')[0]
                n = linenew.split(',')[1].split('[')[0]
                rangefrom = linenew.split(',')[1].split('[')[1]
                rangeto = linenew.split(',')[2]
                addedline = addedline + destig+','+h0+','+n+','+rangefrom+','+rangeto+'\n'
            
            formul_file = open('./config-files/MagParams.dat', 'r', encoding="utf-8")
            records = formul_file.readlines()
            formul_file.close()
            
            newrecords = ''
            for line in records:
                if not line.startswith(destig) and line != '':
                    newrecords = newrecords + line
            newrecords = newrecords + addedline
    
            formul_file = open('./config-files/MagParams.dat', 'w', encoding="utf-8")
            formul_file.write(newrecords)
            formul_file.close()
            koprdial.InfoDialog(0, 'Success', 'Magnitude parameters saved')

    def onActivatedDateR(self):
        ranges = [ str(self.plotopt[0][1].text()), str(self.plotopt[0][2].text()), self.dater, self.plotopt[0][3].currentIndex() ]
        
        if self.working_list:
          matrix = npfunc.RangeConversion(self.kometa, self.returns, self.peridict, self.working_list, ranges, int(self.plotopt[0][4].isChecked()))
          if self.plotopt[0][3].currentIndex() == 0:
            self.plotopt[0][1].setText( matrix[0][self.plotopt[0][3].currentIndex()] )
            self.plotopt[0][2].setText( matrix[1][self.plotopt[0][3].currentIndex()] )
          else:  
            self.plotopt[0][1].setText( str( round(matrix[0][self.plotopt[0][3].currentIndex()],3) ) )
            self.plotopt[0][2].setText( str( round(matrix[1][self.plotopt[0][3].currentIndex()],3) ) )
          self.button_click_update()
          
        else:
          self.plotopt[0][1].setText('')
          self.plotopt[0][2].setText('')
        
        self.dater = self.plotopt[0][3].currentIndex()

    def onActivatedMagR(self):
        if self.working_list:
          self.button_click_update()

    def button_click_setup(self):
        returns0, returns1 = [], []
        for item in self.returns:
          if item[1] == 1:
            returns0.append(item[0])

        tabdialog = setupobs.TabDialog(self.settings, self.types, self.obscode, self.returns)
        tabdialog.exec_()
        self.settings = tabdialog.settings
        self.types = tabdialog.cat
        self.returns = tabdialog.returns
        self.obscode = tabdialog.obscode  
        elements = koprhand.loadcometid( str(self.comlbl.text()) )
        self.UpdatePlot(False)

        # update plot limits if returns changed
        for item in self.returns:
          if item[1] == 1:
            returns1.append(item[0])
        
        if returns0 != returns1:
#          print 'len ', len(self.working_list[0]) # test1
          if len(self.working_list[0]) > 0:
            xlim = npfunc.GetListLim(self.working_list, self.plotopt[0][3].currentIndex(), int(self.plotopt[0][4].isChecked()) )
          else: 
            xlim = npfunc.GetDefaultLim(self.plotopt[0][3].currentIndex(), self.returns, self.peridict)
#            print xlim

          margin = [ datetime.timedelta(days = 10), 10, 0.1, 0.01 ]
          self.sc.ax.set_xlim( ( xlim[0] - margin[self.plotopt[0][3].currentIndex()], xlim[1] + margin[self.plotopt[0][3].currentIndex()]) )
          self.sc.canvas.draw()
          numranges = [ xlim[0], xlim[1], self.plotopt[1][1].text(), self.plotopt[1][2].text() ]
          self.UpdateRanges(numranges, [self.plotopt[0][3].currentIndex()])

    def button_click_update(self):
        elements = koprhand.loadcometid( str(self.comlbl.text()) )
        self.kometa = koprclass.comet()
        self.kometa.LoadComet(str(self.comlbl.text()))
        self.UpdatePlot(False)
    
    def button_click_clear(self):
        text, ok = QtWidgets.QInputDialog.getText(self, 'Comet destignation', 'Comet short name (e.g. 103P or C/2012 X1): ')                
        if text: 
            # updating list of COBS comets from internet and loading elements
            self.peridict, self.returns, self.obscode, self.obs_list, self.working_list = {}, [], [], [], []
            #koprhand.updatecometlist()
            text = text.strip()
            elements = koprhand.loadcometid(text)

            if elements == '':
                koprdial.InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
            else:    
                # clean up fields first
                self.perihelions, self.observers, self.obs_list, self.peridict = [], [], [], {}
                self.plotopt[0][1].setText('')
                self.plotopt[0][2].setText('')
                self.plotopt[1][1].setText('')
                self.plotopt[1][2].setText('')
                
                self.reg.setText('')
                self.adit.setText('')

                self.sc.ax.clear()
                self.sc.ax.grid(True)
                self.sc.fig.tight_layout()  
                self.sc.canvas.draw()
                
                # Load comet data
                self.comlbl.setText(text) # set new comet text
                self.kometa = koprclass.comet()
                self.kometa.LoadComet(str(text))
                output = ''
                for line in self.kometa.magparams:
                    destig, h0, n, strrange_from, strrange_to = line.split(',')
                    output += 'H0={:5s} mag, n={:5s} [{:s}, {:s}]'.format(h0, n, strrange_from, strrange_to.replace('\n',''))+'\n'    
                if output == '': 
                    rest = elements[90:102].split()
                    h0, n = float(rest[0]), float(rest[1])
                    output = 'H0={:5.2f} mag, n={:5.2f} [inf, inf]'.format(h0, n)
                self.reg.setText(output)
                
                try:
                  self.UpdatePlot(True)
                except IOError:            
                  koprdial.InfoDialog(1,'Local data missing','Load data from COBS or MPC first')
                  self.button_load_cobs()
                
    def GrabAditional(self, mp_arrray):
        linelist = [_f for _f in str(self.adit.toPlainText()).split('\n') if _f]
        for line in linelist:
          if line.startswith('H0='):
            linenew = line.replace(' ','').replace('H0=','').replace('mag','').replace('n=','').replace(']','')
            mp_arrray.append([float(linenew.split(',')[0]), float(linenew.split(',')[1].split('[')[0]), float(linenew.split(',')[1].split('[')[1].replace('inf','-9999')), float(linenew.split(',')[2].replace('inf','9999')), '--' ])
        
        if self.settings[11]: # bortle survival limit
          bortle = koprfunc.GetBortleSurvival(self.kometa.elements, self.returns)
          if bortle[1]:
            mp_arrray.append([ bortle[0], 4.0, -9999.0, 9999.0, ':' ])

        return mp_arrray

    def UpdatePlot(self, new):
        # process files to array, and get a jday translations for perihleions
        self.obs_list, self.peridict, self.obscode, self.returns = npfunc.sort( koprhand.ObsFileUpdate( 'local', self.kometa.destig, [] ), self.kometa, new, self.obscode, self.returns)
        if new:
            self.obscode = koprhand.exclude(self.obscode)

        if self.obs_list:
            # limits for all observations
            if new:
              limits = npfunc.getranges(self.obs_list, self.plotopt[0][3].currentIndex())
            else:
              limits = ( str(self.plotopt[0][1].text()), str(self.plotopt[0][2].text()), self.plotopt[1][1].text(), self.plotopt[1][2].text() )
              
            # set filter to select observations ... in case of new comet, ranges are initial limits  
            set_filter = [ self.plotopt[0][3].currentIndex(), limits[0], limits[1] ]

            # change magparams to number array +  grab additional formulas
            mp_arrray = self.GrabAditional(npfunc.magparams(self.kometa.magparams)) 

            # filtered obs list
            corr = [ self.plotopt[1][3].currentIndex(), self.settings[6], self.settings[7], self.settings[5] ]
            array = copy.deepcopy(self.obs_list)
            self.working_list = npfunc.filtered_obs( mp_arrray , array, self.peridict, self.settings, self.types, self.obscode, self.returns, set_filter, corr) 
            
            # get plot lists
            plot_points = npfunc.PlotLists( self.working_list, self.types, self.returns, self.settings[4], int(self.plotopt[0][4].isChecked()), set_filter )
            plot_lines = npfunc.PlotLines( mp_arrray, self.peridict, self.returns, self.settings[4], set_filter, self.plotopt[1][3].currentIndex()) 

            self.sc.ax.clear()
            for line in plot_points:
              self.sc.ax.plot( line[0], line[1], marker=line[2], color=line[3], markersize = 3, ls = 'None' )
    
            for line in plot_lines:
              self.sc.ax.plot( line[0], line[1], color=line[2], lw = 2, ls = line[3], aa = True )
            
            if new:
              numranges = npfunc.NumRanges(set_filter, self.working_list, self.plotopt[0][4].isChecked() )
            else:
              numranges = npfunc.StrNumRanges(set_filter, limits)

            str(self.adit.toPlainText()).split('\n')
            
            import matplotlib.lines as mlines

            legend = []
            for item in str(self.reg.toPlainText()).split('\n'):
              if item.startswith('H0='):
                legend.append(mlines.Line2D([], [], color='black', ls='-', lw = 2, label=item))
            for item in str(self.adit.toPlainText()).split('\n'):
              if item.startswith('H0='):
                legend.append(mlines.Line2D([], [], color='black', ls='--', lw = 2, label=item))

            if self.settings[11]:
              bortle = koprfunc.GetBortleSurvival(self.kometa.elements, self.returns)
              if bortle[1]:
                legend.append(mlines.Line2D([], [], color='black', ls=':', lw = 2, label='Bortle survival limit: {:.2f} mag'.format(round(bortle[0],2)) )) # bortle survival legend
            
            self.sc.ax.legend(handles=legend, fontsize = 8)
              
            margin = [ datetime.timedelta(days = 10), 10, 0.1, 0.01 ]
            self.sc.ax.set_xlim( ( numranges[0] - margin[set_filter[0]], numranges[1] + margin[set_filter[0]]) )
            self.sc.ax.set_ylim( ( numranges[2] + 1, numranges[3] - 1 ) )
            self.sc.ax.grid(True)

            # plot perihelion lines
            returns = {x[0]: x[2] for x in self.returns if x[1] == 1}
            for key in self.peridict:
              if self.peridict[key][0] in list(returns.keys()):
                xcoord = npfunc.GetPlineX(self.peridict[key],self.plotopt[0][3].currentIndex())
                self.sc.ax.plot((xcoord,xcoord), self.sc.ax.get_ylim(), color=returns[self.peridict[key][0]], lw = 2, ls = '-.')
            
            if self.plotopt[0][3].currentIndex() == 0:
              self.sc.fig.autofmt_xdate()
            
            self.sc.ax.set_title('Light curve of: '+self.kometa.name)
            self.sc.ax.set_xlabel(self.plotopt[0][3].currentText(), fontsize=9)
            self.sc.ax.set_ylabel(self.plotopt[1][3].currentText(), fontsize=9)
            self.sc.ax.tick_params(axis='both', which='major', labelsize=8)
            self.sc.fig.tight_layout()  
            self.sc.canvas.draw()
            
            if new:
              self.UpdateRanges(numranges, set_filter)
        else:
            self.sc.ax.clear()
            self.sc.canvas.draw()

    def UpdateRanges(self, numranges, set_filter): # set limits as ranges for new comet                
        if set_filter[0] == 0:
          self.plotopt[0][1].setText( numranges[0].strftime('%Y/%m/%d') )
          self.plotopt[0][2].setText( numranges[1].strftime('%Y/%m/%d') )
        else:
          self.plotopt[0][1].setText( str(numranges[0]) )
          self.plotopt[0][2].setText( str(numranges[1]) )
        self.plotopt[1][1].setText( str(numranges[2]) )
        self.plotopt[1][2].setText( str(numranges[3]) )
                
############################################################################################## buttons
    def button_click_analyse(self):
        if str(self.comlbl.text()) == '':  
            koprdial.InfoDialog(1, 'Errors were encountered', 'Fill all required fields!')  
        elif not self.obs_list:    
            koprdial.InfoDialog(1, 'Errors were encountered', 'Load observations first!')  
        else:   
            limits = ( self.plotopt[0][1].text(), self.plotopt[0][2].text(), self.plotopt[1][1].text(), self.plotopt[1][2].text() )
            # set filter to select observations ... in case of new comet, ranges are initial limits  
            set_filter = [ self.plotopt[0][3].currentIndex(), limits[0], limits[1] ]
            mp_arrray = npfunc.magparams(self.kometa.magparams) # change magparams to number array
            # filtered obs list
            corr = [ 1, self.settings[6], self.settings[7], self.settings[5] ]
            array = copy.deepcopy(self.obs_list)

            self.working_list = npfunc.filtered_obs( mp_arrray , array, self.peridict, self.settings, self.types, self.obscode, self.returns, set_filter, corr) 

            result = npfunc.Analysis(self.working_list[0], self.settings)

            text = ''
            for line in result:
              text += 'H0={:5.2f} mag, n={:5.2f} [{:6.1f}, {:6.1f}]\n'\
                      .format( round(line[0],2), round(line[1],2), round(line[4],1), round(line[5],1) )
              text += '(number:{:5.0f}, +/-:{:5.2f} mag)\n'\
                      .format( line[3], round(line[2],2) )
            self.adit.setText(text)
            self.button_click_update()

    def button_click_percoef(self):
        destig = str(self.comlbl.text())
        if destig == '':  
            koprdial.InfoDialog(1, 'Errors were encountered', 'Fill all required fields!')  
        elif not self.obs_list:    
            koprdial.InfoDialog(1, 'Errors were encountered', 'Load observations first!')  
        else:   
            self.kometa = koprclass.comet()
            self.kometa.LoadComet(str(self.comlbl.text()))
            obs_list = koprhand.FilterObs( self.settings, self.observers, self.perihelions, self.types, self.obs_list, self.kometa, self.lefrom.text(), self.leto.text(), self.cbtyp.currentText() )[0]
            xlist, observers, results, destig = koprfunc.PersonalCoef( obs_list, self.kometa, self.settings[6], self.settings[7] )
            a = ObsSelect(observers, results, ['Calculated personal coeficients','Merge with global data'])
            a.exec_()
            output = koprfunc.ApplyPersonal(xlist, a.output, results, destig)
            a = koprdial.MagDataOutput(output, 'Updated personal coeficients')
            a.exec_()

## *******************************************
############## Dialog classes ################
## *******************************************     

class MPCFilter(QtWidgets.QDialog):
    def __init__(self, perihelions, obslist, kometa, parent = None):
        super(MPCFilter, self).__init__()
        self.obslist, self.kometa, self.fobslist, self.output = obslist, kometa, [], []

        self.elements = [ [ QtWidgets.QLabel('Select return to process:'), QtWidgets.QComboBox(self), QtWidgets.QCheckBox('Total (T)', self), QtWidgets.QCheckBox('Nuclear (N)', self) ],
                          [ QtWidgets.QLabel('% of brightest observations:'), QtWidgets.QLineEdit(self), QtWidgets.QCheckBox('Fixed slope', self), QtWidgets.QLineEdit(self) ] ]

        grid = QtWidgets.QGridLayout()
        for i, line in enumerate(self.elements):
          for j, item in enumerate(line):
             item.setFont(font)
             grid.addWidget(item, i, j)

        self.elements[0][1].setStyleSheet("selection-color: black")
        self.elements[0][2].toggle()
        self.elements[0][3].toggle()
        self.elements[1][2].toggle()
        self.elements[1][1].setText('10')
        self.elements[1][3].setText('4.0')
        self.elements[0][1].activated.connect(self.UpdatePlot)
        self.elements[0][2].stateChanged.connect(self.UpdatePlot)
        self.elements[0][3].stateChanged.connect(self.UpdatePlot)
        datavar.clickable(self.elements[1][1]).connect(self.UpdatePlot)
        self.elements[1][2].stateChanged.connect(self.UpdatePlot)
        datavar.clickable(self.elements[1][3]).connect(self.UpdatePlot)

        for line in perihelions:
            self.elements[0][1].addItem(line)

        intercept, slope, self.fobslist, eline = self.Obslist()
        self.fobslist, tmake, smake = koprfunc.FilterMPCObs( eline, self.fobslist, 10, intercept, 4.0 )
        
        self.sc = koprfig.MPCCanavas(self)
        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)
        self.DataPlot( [[ tmake[0], smake[0] ], [ tmake[1], smake[1] ], [ tmake[2], smake[2] ]] )

        self.sc.ax.set_title('MPC data for: '+self.kometa.name, fontsize=10)
        self.sc.ax.set_ylabel('Mag.', fontsize=9)
        self.sc.ax.set_xlabel('Date', fontsize=9)
        
        btnsave = QtWidgets.QPushButton('Save', self)
        btnsave.setFont(font)
        btnsave.clicked.connect(self.button_click_save)

        btnsaveq = QtWidgets.QPushButton('Save and Quit', self)
        btnsaveq.setFont(font)
        btnsaveq.clicked.connect(self.button_click_saveq)

        btncanc = QtWidgets.QPushButton('Close', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addStretch(1)
        hbox.addWidget(btnsave)
        hbox.addWidget(btnsaveq)
        hbox.addWidget(btncanc)

        vbox = QtWidgets.QVBoxLayout()
        vbox.addLayout(grid)
        vbox.addWidget(self.mpl_toolbar)
        vbox.addWidget(self.sc)
        vbox.addLayout(hbox)
        
        self.setLayout(vbox)  
        self.setWindowTitle('MPC observations')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def DataPlot(self, plots):
        self.sc.ax.plot( plots[0][0], plots[0][1], color='black', lw = 1, aa = True )
        self.sc.ax.plot( plots[1][0], plots[1][1], color='black', marker='*', markersize = 4, ls = 'None', label='Excluded' )
        self.sc.ax.plot( plots[2][0], plots[2][1], color='red', marker='x', markersize = 4, ls = 'None', label='Selected' )
        self.sc.ax.legend(loc=1)
        self.sc.ax.tick_params(labelsize=8)
        self.sc.ax.grid(True)
        self.sc.fig.autofmt_xdate()
        self.sc.fig.tight_layout()
        self.sc.canvas.draw()

    def UpdatePlot(self):
        try:
          perc = float(self.elements[1][1].text())
          intercept, slope, self.fobslist, eline = self.Obslist()
          self.fobslist, tmake, smake = koprfunc.FilterMPCObs( eline, self.fobslist, perc, intercept, slope )
          self.sc.ax.clear()
          self.DataPlot( [[ tmake[0], smake[0] ], [ tmake[1], smake[1] ], [ tmake[2], smake[2] ]] )
        except ValueError:
          pass
        
    def Obslist(self):
        slope = 0
        if self.elements[1][2].isChecked(): 
          slope = float(self.elements[1][3].text())
        return koprfunc.MPCComParam(str(self.elements[0][1].currentText())+'/07/01.00', self.kometa, self.obslist, slope, self.elements[0][2].isChecked(), self.elements[0][3].isChecked())

    def button_click_save(self):
        self.output += self.fobslist

    def button_click_saveq(self):
        self.output += self.fobslist
        self.close()

    def button_click_cancel(self):
        self.close()

class ScrollAnalyzer(QtWidgets.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtWidgets.QScrollArea.__init__(self, parent)

        self.w = Analyzer(self)
        self.setWidget(self.w)    
